"use strict";
const jsonStringify = (data, opts) => {
    let comparator;
    let allowCycle = false;
    if (typeof opts === "function") {
        comparator = opts;
    }
    else {
        allowCycle = (opts === null || opts === void 0 ? void 0 : opts.cycles) === true;
        comparator = opts === null || opts === void 0 ? void 0 : opts.cmp;
    }
    const seenObjects = new Set();
    return (function stringify(node) {
        if (node && node.toJSON && typeof node.toJSON === "function") {
            node = node.toJSON();
        }
        if (node === undefined) {
            return;
        }
        if (node === null) {
            return "null";
        }
        if (typeof node === "number") {
            return isFinite(node) ? "" + node : "null";
        }
        if (typeof node !== "object") {
            return JSON.stringify(node);
        }
        if (Array.isArray(node)) {
            return `[${node.map((item) => { var _a; return (_a = stringify(item)) !== null && _a !== void 0 ? _a : "null"; }).join(",")}]`;
        }
        if (seenObjects.has(node)) {
            if (allowCycle)
                return JSON.stringify("__cycle__");
            throw new TypeError("Converting circular structure to JSON");
        }
        seenObjects.add(node);
        const sortedKeys = comparator
            ? Object.entries(node)
                .sort((a, b) => comparator({
                key: a[0],
                value: a[1],
            }, {
                key: b[0],
                value: b[1],
            }))
                .map((keyValue) => keyValue[0])
            : Object.keys(node).sort();
        const objKeyValues = [];
        for (const key of sortedKeys) {
            const value = stringify(node[key]);
            if (!value) {
                continue;
            }
            objKeyValues.push(`"${key}":${value}`);
        }
        seenObjects.delete(node);
        return `{${objKeyValues.join(",")}}`;
    })(data);
};
jsonStringify.default = jsonStringify;
module.exports = jsonStringify;
//# sourceMappingURL=index.js.map